@@ -16,7 +16,7 @@ from django_caldav.acl import FullAcl
 from django_caldav.lock import DummyLock
 from django_caldav.models import CalDavEvent
 from django_caldav.resources import CalDavResource
-from django_caldav.utils import WebDAV, CalDAV, url_join, CalDAV_MAP, WebDAV_MAP, CalendarServer_MAP, CalendarServer
+from django_caldav.utils import WebDAV, CalDAV, url_join, CalDAV_MAP, WebDAV_MAP, CalendarServer_MAP, CalendarServer, iCalendar
 
 
 class CalDavFeedView(ICalFeed):
@@ -46,9 +46,8 @@ class CalDavFeedView(ICalFeed):
     def item_guid(self, item):
         return item.pk
 
-    @staticmethod
-    def item_save(base_item, iCalendar_component):
-        return HttpResponseForbidden()
+    def item_save(self, request, base_item, iCalendar_component, *args, **kwargs):
+        return None
 
 
 class CalDavView(DavView):
@@ -101,7 +100,7 @@ class CalDavView(DavView):
             "xml": self.get_request_body_as_etree_with_ns_map(request)
         }
         print(self.request_body["plain"])
-        return super(CalDavView, self).dispatch(request, path)
+        return super(CalDavView, self).dispatch(request, path, *args, **kwargs)
 
     def get_request_body_as_etree_with_ns_map(self, request):
         try:
@@ -439,31 +438,42 @@ class CalDavView(DavView):
             feed_request.path = calendar_uri
             feed_request.is_secure = request.is_secure
             feed_request.path = request.path
-            feed_response = feed.__call__(request).content
+            feed_request.user = request.user
+            feed_response = feed.__call__(feed_request, *args, **kwargs).content
             feed_response = feed_response.replace("\r", "")
             if not calendar_query:
                 events = CalDavFeedView.one_event_per_calendar(feed_response)
             else:
                 events = [(calendar_uri, smart_unicode(feed_response))]
 
+            multistatus = []
+            for event in events:
+                calendar = Calendar.from_ical(event[1])
+                etag = None
+                for component in calendar.walk():
+                    if component.name == "VEVENT":
+                        etag = iCalendar.unicode(component.get("UID"))
+                        break
+                multistatus.append(
+                    WebDAV(
+                        "response",
+                        WebDAV("href", event[0]),
+                        WebDAV(
+                            "propstat",
+                            WebDAV(
+                                "prop",
+                                CalDAV("calendar-data", event[1])
+                            ),
+                            WebDAV("getetag", "\"{etag}\"".format(etag=etag))
+                        ),
+                        WebDAV("status", "HTTP/1.1.200.OK")
+                    )
+                )
+
             responses.append(
                 WebDAV(
                     "multistatus",
-                    *[
-                        WebDAV(
-                            "response",
-                            WebDAV("href", event[0]),
-                            WebDAV(
-                                "propstat",
-                                WebDAV(
-                                    "prop",
-                                    CalDAV("calendar-data", event[1])
-                                ),
-                                WebDAV("getetag", "\"{etag}\"".format(etag=CalDavView.getECTag(event[1])))
-                            ),
-                            WebDAV("status", "HTTP/1.1.200.OK")
-                        ) for event in events
-                    ]
+                    *multistatus
                 )
             )
 
@@ -474,7 +484,7 @@ class CalDavView(DavView):
         return response
 
     def put(self, request, path, xbody=None, *args, **kwargs):
-        calendar = Calendar.from_ical(self.request_body["plain"])
+        calendar = Calendar.from_ical(smart_unicode(self.request_body["plain"]))
         for component in calendar.walk():
             if component.name == "VEVENT":
                 base_item = CalDavEvent()
@@ -485,5 +495,13 @@ class CalDavView(DavView):
                 base_item.location = component.get("LOCATION")
                 base_item.description = component.get("DESCRIPTION")
                 base_item.finalize()
-                return self.feed_view.item_save(base_item, component)
+
+                item = self.feed_view().item_save(request, base_item, component, *args, **kwargs)
+                if item:
+                    response = HttpResponse()
+                    response.status_code = 201
+                    response["ETag"] = u"\"{UID}\"".format(UID=self.feed_view().item_guid(item))
+                    return response
+                else:
+                    return HttpResponseBadRequest()
         return HttpResponseBadRequest()
\ No newline at end of file
